local Graph = AceLibrary("Graph-1.0")
local me={}

local RowHeight=14
function RecountSortFunc(a,b)
	if a[2]>b[2] then
		return true
	end
	return false
end

function Recount:SelectUpperDetailTable(num)
	if num==nil or (Recount.DetailWindow.UpperTable and num>table.maxn(Recount.DetailWindow.UpperTable)) then
		return
	end

	local Rows=Recount.DetailWindow.TopRows

	for i=1,8 do
		Rows[i].Background:Hide()
	end

	if num<=8 then
		Rows[num].Background:Show()
	end

	Recount.DetailWindow.Selected=num
	if Recount.DetailWindow.UpperTable and Recount.DetailWindow.UpperTable[num] and Recount.DetailWindow.UpperTable[num][3] then
		Recount:FillLowerDetailTable(Recount.DetailWindow.UpperTable[num][3])
	end
end

function me:SelectLowerDetailTable(num)
	if num==nil then
		return
	end

	local Rows=Recount.DetailWindow.BotRows

	for i=1,8 do
		Rows[i].Background:Hide()
	end

	if num<=8 then
		Rows[num].Background:Show()
	end
end

function Recount:FillUpperDetailTable(Data)
	local UpperTable={}
	local total=0
	for k,v in pairs(Data) do
		table.insert(UpperTable,{k,v.amount,v.Details})
		total=total+v.amount
	end

	table.sort(UpperTable,RecountSortFunc)

	local PieChart=Recount.DetailWindow.TopPieChart

	PieChart:ResetPie()

	local MaxNum=table.maxn(UpperTable)

	for k,v in pairs(UpperTable) do
		v[4]=100*v[2]/total

		if k~=MaxNum then
			v[5]=PieChart:AddPie(v[4])
		else
			v[5]=PieChart:CompletePie()
		end
	end

	for i=1,8 do
		local entry=UpperTable[i]
		local row=Recount.DetailWindow.TopRows[i]
		if entry then
			row.Background:SetVertexColor((entry[5][1]+0.5)/2,(entry[5][2]+0.5)/2,(entry[5][3]+0.5)/2,0.3)
			row.Key:SetVertexColor(entry[5][1],entry[5][2],entry[5][3],1.0)
			row.Name:SetText(entry[1])
			row.Amount:SetText(entry[2])
			row.Percent:SetText(math.floor(entry[4]+0.5).."%")
			row.Data=entry[3]
			row:Show()
		else
			row:Hide()
		end
	end
	
	Recount.DetailWindow.UpperTable=UpperTable
	if Recount.DetailWindow.Selected and UpperTable[Recount.DetailWindow.Selected] and UpperTable[Recount.DetailWindow.Selected][3] then
		Recount:FillLowerDetailTable(UpperTable[Recount.DetailWindow.Selected][3])
	end
	Recount.DetailWindow:Show()
end


local HitTypeColors={
	Hit={0.0,0.9,0.0},
	Crit={0.9,0.0,0.0},
	Tick={1.0,0.5,0.0},
	Miss={0.0,1.0,1.0},
	Dodge={0.5,0.0,1.0},
	Parry={0.5,0.5,0.5},
	Resist={0.0,0.5,0.5},
	Immune={0.5,0.0,0.0},
	Block={0.0,0.5,0.0},
	Glancing={0.5,1.0,0.5},
	Crushing={0.5,0.0,0.5},
	Absorb={0.25,0.25,0.25},
}
function Recount:FillLowerDetailTable(Data)
	local dispTable={}
	local total=0
	for k,v in pairs(Data) do
		local avg

		if v.amount then
			avg=math.floor(v.amount/v.count+0.5)
		end
		table.insert(dispTable,{k,v.count,v.min,avg,v.max})
		total=total+v.count
	end

	table.sort(dispTable,RecountSortFunc)

	local PieChart=Recount.DetailWindow.BotPieChart

	PieChart:ResetPie()

	local MaxNum=table.maxn(dispTable)

	for k,v in pairs(dispTable) do
		v[6]=100*v[2]/total
		v[7]=HitTypeColors[v[1]]

		if k~=MaxNum then
			v[7]=PieChart:AddPie(v[6],v[7])
		else
			v[7]=PieChart:CompletePie(v[7])
		end
	end

	for i=1,8 do
		local entry=dispTable[i]
		local row=Recount.DetailWindow.BotRows[i]
		if entry then
			row.Key:SetVertexColor(entry[7][1],entry[7][2],entry[7][3],1.0)
			row.Background:SetVertexColor((entry[7][1]+0.5)/2,(entry[7][2]+0.5)/2,(entry[7][3]+0.5)/2,0.3)
			row.Name:SetText(entry[1])
			row.Count:SetText(entry[2])
			if entry[3] then
				row.Min:SetText(entry[3])
				row.Min:Show()
			else
				row.Min:Hide()
			end

			if entry[4] then
				row.Avg:SetText(entry[4])
				row.Avg:Show()
			else
				row.Avg:Hide()
			end

			if entry[5] then
				row.Max:SetText(entry[5])
				row.Max:Show()
			else
				row.Max:Hide()
			end

			row.Percent:SetText(math.floor(entry[6]+0.5).."%")
			row.Data=entry[3]
			row:Show()
		else
			row:Hide()
		end
	end
end


--The titles are slightly different for various modes
--.TopNames = Names of the entries for the top data
--.TopAmount = What we call the amount for the top
--.BotNames = Names of the entries for the bottom
--.BotMin = The minimum label for bottom
--.BotAvg = The average label for bottom
--.BotMax = The minimum label for bottom
--.BotAmount = Label for what the amount is on the bottom
function Recount:SetupDetailTitles(ForWho,MainTitle,Titles)

	Recount.DetailWindow.Title:SetText("Detail Window - "..ForWho..MainTitle)
	local Labels=Recount.DetailWindow.TopRowLabels
	Labels.Name:SetText(Titles.TopNames)
	Labels.Amount:SetText(Titles.TopAmount)

	Labels=Recount.DetailWindow.BotRowLabels
	Labels.Name:SetText(Titles.BotNames)
	Labels.Min:SetText(Titles.BotMin)
	Labels.Avg:SetText(Titles.BotAvg)
	Labels.Max:SetText(Titles.BotMax)
	Labels.Count:SetText(Titles.BotAmount)
end

--Create Detail Window Function
function Recount:CreateDetailWindow()
	Recount.DetailWindow=CreateFrame("Frame","Recount_DetailWindow",UIParent)

	local theFrame=Recount.DetailWindow

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(320)
	theFrame:SetWidth(450)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\Recount\\textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,0.0,0.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)

	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)

	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
					
	theFrame:SetScript("OnHide", function()
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetPoint("TOPLEFT",theFrame,"TOPLEFT",6,-15)
	theFrame.Title:SetTextColor(1.0,1.0,1.0)
	theFrame.Title:SetText("Detail Window - Hostile Abilites")

	theFrame.TopPieChart=Graph:CreateGraphPieChart("Recount_DetailWindow_TopPieChart",theFrame,"LEFT","LEFT",0,56.5,150,150)
	theFrame.BotPieChart=Graph:CreateGraphPieChart("Recount_DetailWindow_BotPieChart",theFrame,"LEFT","LEFT",0,-88,150,150)

	theFrame.TopPieChart:SetSelectionFunc(Recount.SelectUpperDetailTable)
	theFrame.BotPieChart:SetSelectionFunc(me.SelectLowerDetailTable)
	
	theFrame.TopRowLabels=CreateFrame("FRAME",nil,theFrame)
	local Labels=theFrame.TopRowLabels

	Labels:SetPoint("TOPLEFT",theFrame,"TOP",-70,-32)
	Labels:SetWidth(285)
	Labels:SetHeight(RowHeight)

	Labels.Key=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Key:SetPoint("LEFT",Labels,"LEFT",2,0)
	Labels.Key:SetText("K")

	Labels.Count=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Count:SetPoint("LEFT",Labels,"LEFT",16,0)
	Labels.Count:SetText("#")

	Labels.Name=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Name:SetPoint("LEFT",Labels,"LEFT",30,0)
	Labels.Name:SetText("Name of Ability")

	Labels.Amount=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Amount:SetPoint("RIGHT",Labels,"RIGHT",-40,0)
	Labels.Amount:SetText("Damage")

	Labels.Percent=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Percent:SetPoint("RIGHT",Labels,"RIGHT",-8,0)
	Labels.Percent:SetText("%")

	theFrame.TopRows={}
	
	for i=1,8 do
		local Row=CreateFrame("FRAME",nil,theFrame)

		Row.id=i
		Row:EnableMouse(true)
		Row:SetScript("OnEnter", function() Recount:SelectUpperDetailTable(this.id) end)

		Row:SetWidth(285)
		Row:SetHeight(RowHeight)
		Row:SetPoint("TOPLEFT",theFrame,"TOP",-70,-32-(RowHeight+2)*i)

		Row.Background=Row:CreateTexture(nil,"BACKGROUND")
		Row.Background:SetAllPoints(Row)
		Row.Background:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		Row.Background:Hide()

		Row.Key=Row:CreateTexture(nil,"OVERLAY")
		Row.Key:SetPoint("LEFT",Row,"LEFT",0,0)
		Row.Key:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		Row.Key:SetWidth(12)
		Row.Key:SetHeight(12)

		Row.Count=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Count:SetPoint("LEFT",Row.Key,"LEFT",16,0)
		Row.Count:SetText(i)
		Row.Count:SetTextColor(1.0,1.0,1.0,1.0)
		
		Row.Name=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Name:SetPoint("LEFT",Row,"LEFT",30,0)
		Row.Name:SetText("Test")
		Row.Name:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Amount=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Amount:SetPoint("RIGHT",Row,"RIGHT",-40,0)
		Row.Amount:SetText("12345")
		Row.Amount:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Percent=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Percent:SetPoint("RIGHT",Row,"RIGHT",0,0)
		Row.Percent:SetText("10%")
		Row.Percent:SetTextColor(1.0,1.0,1.0,1.0)


		theFrame.TopRows[i]=Row
	end

	local Halfway=(theFrame:GetHeight()-32)/2
	Graph:DrawLine(theFrame,2,Halfway,theFrame:GetWidth()-2,Halfway,24,{0.6,0.0,0.0,1.0},"ARTWORK")


	theFrame.BotRowLabels=CreateFrame("FRAME",nil,theFrame)
	local Labels=theFrame.BotRowLabels

	Labels:SetPoint("TOPLEFT",theFrame,"TOP",-70,-32-Halfway)
	Labels:SetWidth(285)
	Labels:SetHeight(RowHeight)

	Labels.Key=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Key:SetPoint("LEFT",Labels,"LEFT",2,0)
	Labels.Key:SetText("K")

	Labels.Pos=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Pos:SetPoint("LEFT",Labels,"LEFT",16,0)
	Labels.Pos:SetText("#")

	Labels.Name=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Name:SetPoint("LEFT",Labels,"LEFT",30,0)
	Labels.Name:SetText("Type")

	Labels.Min=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Min:SetPoint("RIGHT",Labels,"RIGHT",-160,0)
	Labels.Min:SetText("Min")

	Labels.Avg=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Avg:SetPoint("RIGHT",Labels,"RIGHT",-120,0)
	Labels.Avg:SetText("Avg")

	Labels.Max=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Max:SetPoint("RIGHT",Labels,"RIGHT",-80,0)
	Labels.Max:SetText("Max")

	Labels.Count=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Count:SetPoint("RIGHT",Labels,"RIGHT",-40,0)
	Labels.Count:SetText("Count")

	Labels.Percent=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Percent:SetPoint("RIGHT",Labels,"RIGHT",-8,0)
	Labels.Percent:SetText("%")

	theFrame.BotRows={}
	
	for i=1,8 do
		local Row=CreateFrame("FRAME",nil,theFrame)

		Row.id=i
		Row:EnableMouse(true)
		Row:SetScript("OnEnter", function() me:SelectLowerDetailTable(this.id) end)

		Row:SetWidth(285)
		Row:SetHeight(RowHeight)
		Row:SetPoint("TOPLEFT",theFrame,"TOP",-70,-32-Halfway-(RowHeight+2)*i)

		Row.Background=Row:CreateTexture(nil,"BACKGROUND")
		Row.Background:SetAllPoints(Row)
		Row.Background:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		Row.Background:Hide()

		Row.Key=Row:CreateTexture(nil,"OVERLAY")
		Row.Key:SetPoint("LEFT",Row,"LEFT",0,0)
		Row.Key:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		Row.Key:SetWidth(12)
		Row.Key:SetHeight(12)

		Row.Pos=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Pos:SetPoint("LEFT",Row.Key,"LEFT",16,0)
		Row.Pos:SetText(i)
		Row.Pos:SetTextColor(1.0,1.0,1.0,1.0)
		
		Row.Name=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Name:SetPoint("LEFT",Row,"LEFT",30,0)
		Row.Name:SetText("Test")
		Row.Name:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Min=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Min:SetPoint("RIGHT",Row,"RIGHT",-160,0)
		Row.Min:SetText("32")
		Row.Min:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Avg=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Avg:SetPoint("RIGHT",Row,"RIGHT",-120,0)
		Row.Avg:SetText("32")
		Row.Avg:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Max=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Max:SetPoint("RIGHT",Row,"RIGHT",-80,0)
		Row.Max:SetText("32")
		Row.Max:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Count=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Count:SetPoint("RIGHT",Row,"RIGHT",-40,0)
		Row.Count:SetText("32")
		Row.Count:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Percent=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Percent:SetPoint("RIGHT",Row,"RIGHT",0,0)
		Row.Percent:SetText("10%")
		Row.Percent:SetTextColor(1.0,1.0,1.0,1.0)


		theFrame.BotRows[i]=Row
	end

	theFrame.CloseButton=CreateFrame("Button",nil,theFrame)
	theFrame.CloseButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
	theFrame.CloseButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
	theFrame.CloseButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.CloseButton:SetWidth(20)
	theFrame.CloseButton:SetHeight(20)
	theFrame.CloseButton:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-4,-12)
	theFrame.CloseButton:SetScript("OnClick",function() this:GetParent():Hide() end)

	theFrame.LeftButton=CreateFrame("Button",nil,theFrame)
	theFrame.LeftButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up.blp")
	theFrame.LeftButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down.blp")
	theFrame.LeftButton:SetWidth(16)
	theFrame.LeftButton:SetHeight(18)
	theFrame.LeftButton:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-40,-12)
	theFrame.LeftButton:SetScript("OnClick",function() Recount:DetailWindowPrevMode() end)

	theFrame.RightButton=CreateFrame("Button",nil,theFrame)
	theFrame.RightButton:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up.blp")
	theFrame.RightButton:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down.blp")	
	theFrame.RightButton:SetWidth(16)
	theFrame.RightButton:SetHeight(18)
	theFrame.RightButton:SetPoint("LEFT",theFrame.LeftButton,"RIGHT",1,0)
	theFrame.RightButton:SetScript("OnClick",function() Recount:DetailWindowNextMode() end)

	theFrame:Hide()
end