Recount = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0")
Recount:RegisterDB("RecountDB","RecountPerCharDB")

local Roster = AceLibrary("Roster-2.1")
local Parser = ParserLib:GetInstance("1.1");

local DataVersion = 5

Recount.consoleOptions = {
	type = 'group',
	args = {
		reset = {
			name = "Reset",
			desc = "Resets the data",
			type = 'execute',
			func = function() Recount.db.char.combatants={};Recount.db.char.CombatTimes={} end
		},
		show = {
			name = "Show",
			desc = "Shows the main window",
			type = 'execute',
			func = function() Recount.MainWindow:Show() end
		}
	}
}

Recount:RegisterChatCommand({"/recount"}, Recount.consoleOptions)

function Recount:FindUnit(name)
	for unit in Roster:IterateRoster(true) do
		if unit.name==name then
			return unit.unitid
		elseif UnitName(unit.unitid.."target")==name then
			return unit.unitid.."target"
		end
	end
	return nil
end

function Recount:InitFightData(data)
	--Init Data tracked
	data.Damage=0
	data.FDamage=0
	data.DamageTaken=0
	data.Healing=0
	data.HealingTaken=0
	data.Overhealing=0

	--Ability Data
	data.Attacks={}
	data.FAttacks={}
	data.Heals={}
	data.OverHeals={}	

	--Interaction Data
	data.DamagedWho={} --Who did I damage?
	data.FDamagedWho={} --Who did I damage?
	data.WhoDamaged={} --Who damaged me?
	data.HealedWho={} --Who did I heal?
	data.WhoHealed={} --Who healed me?

	--Elemental Tracking
	data.ElementDone={}
	data.ElementDoneResist={}
	data.ElementTaken={}
	data.ElementTakenResist={}
end

function Recount:AddCombatant(name)
	Recount.db.char.combatants[name]={}

	local combatant=Recount.db.char.combatants[name]

	local unit=Recount:FindUnit(name)
	if unit then
		if Roster:GetUnitIDFromName(name) then
			combatant.inGroup=true
		else
			combatant.inGroup=false
		end
		combatant.isFriend=UnitIsFriend("player",unit)
		combatant.class, combatant.enClass=UnitClass(unit)
		combatant.unit=unit
		combatant.level=UnitLevel(unit)
	else
		combatant.isFriend=false
		combatant.enClass="WARRIOR"
		combatant.checkLater=true
		combatant.inGroup=false
		combatant.level=0
	end
	
	combatant.OverallData={}
	combatant.CurrentFightData={}
	combatant.LastFightData={}

	Recount:InitFightData(combatant.OverallData)
	Recount:InitFightData(combatant.CurrentFightData)
	Recount:InitFightData(combatant.LastFightData)
	

	combatant.TimeWindows={}
	local TimeWindows=combatant.TimeWindows
	
	TimeWindows.Damage={}
	TimeWindows.DamageTaken={}
	TimeWindows.Healing={}
	TimeWindows.Overhealing={}
	TimeWindows.HealingTaken={}

	for i=1,25 do
		TimeWindows.Damage[i]=0
		TimeWindows.DamageTaken[i]=0
		TimeWindows.Healing[i]=0
		TimeWindows.Overhealing[i]=0
		TimeWindows.HealingTaken[i]=0
	end

	combatant.TimeData={}
	local TimeData=combatant.TimeData

	TimeData.Damage={}
	TimeData.DamageTaken={}
	TimeData.Healing={}
	TimeData.Overhealing={}
	TimeData.HealingTaken={}

	combatant.TimeNeedZero={}
end

--.MainTitle = What you see on the window title 
--.TopNames = Names of the entries for the top data
--.TopAmount = What we call the amount for the top
--.BotNames = Names of the entries for the bottom
--.BotMin = The minimum label for bottom
--.BotAvg = The average label for bottom
--.BotMax = The minimum label for bottom
--.BotAmount = Label for what the amount is on the bottom
local DetailTitles={}
DetailTitles.Attacks={
	TopNames = "Ability Name",
	TopAmount = "Damage",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

DetailTitles.DamagedWho={
	TopNames = "Player/Mob Name",
	TopAmount = "Damage",
	BotNames = "Attack Name",
	BotMin = "",
	BotAvg = "",
	BotMax = "",
	BotAmount = "Damage"
}

DetailTitles.Heals={
	TopNames = "Heal Name",
	TopAmount = "Heal",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

DetailTitles.HealedWho={
	TopNames = "Player/Mob Name",
	TopAmount = "Healed",
	BotNames = "Heal Name",
	BotMin = "",
	BotAvg = "",
	BotMax = "",
	BotAmount = "Healed"
}

DetailTitles.OverHeals={
	TopNames = "Heal Name",
	TopAmount = "Overheal",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

local DataModes={}

function DataModes:DamageReturner(data)
	return (data[Recount.CurDataSet].Damage or 0), {{data[Recount.CurDataSet].Attacks,"'s Hostile Attacks",DetailTitles.Attacks},{data[Recount.CurDataSet].DamagedWho," Damaged Who",DetailTitles.DamagedWho}}
end

function DataModes:FriendlyDamageReturner(data)
	return (data[Recount.CurDataSet].FDamage or 0), {{data[Recount.CurDataSet].FAttacks,"'s Friendly Fire",DetailTitles.Attacks},{data[Recount.CurDataSet].FDamagedWho," Friendly Fired On",DetailTitles.DamagedWho}}
end

function DataModes:DamageTakenReturner(data)
	return (data[Recount.CurDataSet].DamageTaken or 0), {{data[Recount.CurDataSet].WhoDamaged," Took Damage From",DetailTitles.DamagedWho}}
end

function DataModes:HealingReturner(data)
	return (data[Recount.CurDataSet].Healing or 0), {{data[Recount.CurDataSet].Heals,"'s Effective Healing",DetailTitles.Heals},{data[Recount.CurDataSet].HealedWho," Healed Who",DetailTitles.HealedWho},{data[Recount.CurDataSet].OverHeals,"'s Overhealing",DetailTitles.OverHeals}}
end

function DataModes:HealingTaken(data)
	return (data[Recount.CurDataSet].HealingTaken or 0), {{data[Recount.CurDataSet].WhoHealed," was Healed by",DetailTitles.HealedWho}}
end

function DataModes:OverhealingReturner(data)
	return (data[Recount.CurDataSet].Overhealing or 0), {{data[Recount.CurDataSet].OverHeals,"'s Overhealing",DetailTitles.OverHeals},{data[Recount.CurDataSet].Heals,"'s Effective Healing",DetailTitles.Heals},{data[Recount.CurDataSet].HealedWho," Healed Who",DetailTitles.HealedWho}}
end



function Recount:AddSortedTooltipData(title,data,num)
	local SortedData={}
	GameTooltip:AddLine(title,1,0.82,0)

	total=0
	for k,v in pairs(data) do
		table.insert(SortedData,{k,v.amount})
		total=total+v.amount
	end

	table.sort(SortedData,RecountSortFunc)

	for i=1,3 do
		if SortedData[i] then
			GameTooltip:AddDoubleLine(i..". "..SortedData[i][1],SortedData[i][2].." ("..math.floor(100*SortedData[i][2]/total).."%)",1,1,1,1,1,1)
		end
	end

end
--The various tooltip functions used for each of the main window data displays
local TooltipFuncs={}
function TooltipFuncs:Damage(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Damage Abilties",data[Recount.CurDataSet].Attacks,3)
	GameTooltip:AddLine("")
	Recount:AddSortedTooltipData("Top 3 Attacked",data[Recount.CurDataSet].DamagedWho,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end

function TooltipFuncs:FDamage(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Friendly Attacks",data[Recount.CurDataSet].FAttacks,3)
	GameTooltip:AddLine("")
	Recount:AddSortedTooltipData("Top 3 Friendly Fired On",data[Recount.CurDataSet].FDamagedWho,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end

function TooltipFuncs:DamageTaken(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Attacked by",data[Recount.CurDataSet].WhoDamaged,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end

function TooltipFuncs:Healing(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Heals",data[Recount.CurDataSet].Heals,3)
	GameTooltip:AddLine("")
	Recount:AddSortedTooltipData("Top 3 Healed",data[Recount.CurDataSet].HealedWho,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end

function TooltipFuncs:HealingTaken(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Healed By",data[Recount.CurDataSet].WhoHealed,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end

function TooltipFuncs:Overhealing(name,data)
	local SortedData,total
	GameTooltip:ClearLines()
	GameTooltip:AddLine(name)
	Recount:AddSortedTooltipData("Top 3 Over Heals",data[Recount.CurDataSet].OverHeals,3)
	GameTooltip:AddLine("<Click for more Details>",0,0.9,0)
end



local MainWindowModes={
{"Damage Done",DataModes.DamageReturner,TooltipFuncs.Damage},
{"Friendly Fire",DataModes.FriendlyDamageReturner,TooltipFuncs.FDamage},
{"Damage Taken",DataModes.DamageTakenReturner,TooltipFuncs.DamageTaken},
{"Healing Done",DataModes.HealingReturner,TooltipFuncs.Healing},
{"Healing Taken",DataModes.HealingTaken,TooltipFuncs.HealingTaken},
{"Overhealing Done",DataModes.OverhealingReturner,TooltipFuncs.Overhealing}}

function Recount:RosterLib_UnitChanged()
	for k,v in pairs(Recount.db.char.combatants) do
		local unitid = Roster:GetUnitIDFromName(k)

		--Must be in our group
		if unitid then
			v.unit=unitid
			v.inGroup=true
		else
			v.inGroup=false
		end
	end
end

local TimeWeight = 1/25
function Recount:TimeTick()
	local Time=GetTime()
	local TimeFormatted

	--First check if combat status changed
	Recount:CheckCombat(Time)

	--Need to increment where data gets put and erase the old ones
	Recount.TimeStep=Recount.TimeStep+1
	if Recount.TimeStep>25 then
		Recount.TimeStep=1
	end

	for name,v in pairs(Recount.db.char.combatants) do		
		for k, v2 in pairs(v.TimeWindows) do
			local NewEntry=0

			for _,v3 in pairs(v2) do
				NewEntry=NewEntry+v3
			end
			
			if NewEntry>0 then
				--do we need a leading zero?
				if not v.TimeNeedZero[k] then
					table.insert(v.TimeData[k],{Time-1, 0})
					v.TimeNeedZero[k]=true
				end
				NewEntry=NewEntry*TimeWeight
				table.insert(v.TimeData[k],{Time, NewEntry})
			elseif v.TimeNeedZero[k] then --Check if we need a trailing zero
				table.insert(v.TimeData[k],{Time, 0})
				v.TimeNeedZero[k]=false
			end
			--Need to set where we will be putting new data to 0
			v2[Recount.TimeStep]=0
		end

		--Lets see if this unit needs to be updated		
		if v.checkLater then
			local Unit=Recount:FindUnit(name)
			if Unit then
				v.isFriend=UnitIsFriend("player",Unit)
				v.class, v.enClass=UnitClass(Unit)
				v.checkLater=false
				v.level=UnitLevel(Unit)
			end
		end		
	end
end

function Recount:CheckCombat(Time)
	local TimeFormatted
	for unit in Roster:IterateRoster(true) do
		if UnitAffectingCombat(unit.unitid) then
			if not Recount.InCombat then	
				local Hour,Min,Seconds
				Hour,Min=GetGameTime()
				Seconds=math.fmod(time(),60)
				TimeFormatted=string.format("%02d:%02d:%02d",Hour,Min,Seconds)

				Recount.InCombat=true
				Recount.InCombatT=Time
				Recount.InCombatF=TimeFormatted
				Recount.FightingWho=""
				Recount.FightingLevel=0

			end
			--Want to return if we have someone in combat since don't want to get out yet
			return
		end
	end

	if Recount.InCombat then --We were in combat but no longer
		local Hour,Min,Seconds
		Hour,Min=GetGameTime()
		Seconds=math.fmod(time(),60)
		TimeFormatted=string.format("%02d:%02d:%02d",Hour,Min,Seconds)

		table.insert(Recount.db.char.CombatTimes,{Recount.InCombatT,Time,Recount.InCombatF,TimeFormatted,Recount.FightingWho})
		Recount.InCombat=false

		--Save current data as the last fight
		for name,v in pairs(Recount.db.char.combatants) do
			v.LastFightData=v.CurrentFightData
			v.CurrentFightData={}
			Recount:InitFightData(v.CurrentFightData)
		end
	end
end

function Recount:OnEnable()	
	if Recount.db.char.combatants==nil then
		Recount.db.char.combatants={}
		Recount.db.char.CombatTimes={}
		Recount.db.char.version=DataVersion
	end
		
	if Recount.db.char.version~=DataVersion then
		Recount.db.char.combatants={}
		Recount.db.char.CombatTimes={}
		Recount.db.char.version=DataVersion
	end


	Parser:RegisterInfoType("Recount","hit",Recount_ParseDamageEvent)
	Parser:RegisterInfoType("Recount","miss",Recount_ParseMissEvent)
	Parser:RegisterInfoType("Recount","heal",Recount_ParseHealEvent)
	
	Recount.TimeStep=1
	Recount.InCombat=false
	Recount.CurDataSet="OverallData"

	Recount:CreateMainWindow()
	Recount:CreateDetailWindow()
	Recount:CreateGraphWindow()
	Recount:LoadMainWindowData(MainWindowModes)	
	Recount:ScheduleRepeatingEvent(Recount.TimeTick,1)

	Recount:RegisterEvent("RosterLib_UnitChanged")	
end